//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "global2.h"
#include "NymGenRBDlgs.h"
#include "NymPrevDlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNymGenRBDlg *NymGenRBDlg;
const AnsiString directives =
  ",Anon-To:,Anon-Post-To:,Remix-To:,Encrypt-To:,Remail-To:,Test-To:,Encrypt-Test:,"
   "Null:,Rand-Hop:,Latent-Time:,Cutmarks:,Encrypt-Key:,Encrypt-3DES:,Encrypt-CAST:,"
   "Encrypt-Subject:,Inflate:,Max-Size:,Max-Count:,Max-Date:,Encrypted:,";
const AnsiString headers =
  ",Subject:,Newsgroups:,Followup-To:,From:,CC:,Cc:,Bcc:,Reply-To:,";
//---------------------------------------------------------------------------

bool __fastcall GenerateReplyBlocks(TForm *Owner, TStrings *Blocks, TPGPsdk *PGP, bool Preview)
{
   assert(Owner != NULL && Blocks != NULL);

   NymGenRBDlg = new TNymGenRBDlg(Owner, Blocks, PGP, Preview);

   NymGenRBDlg->ShowModal();
   int mrResult = NymGenRBDlg->ModalResult;
   if (mrResult == mrOk)
      Blocks->Text = NymGenRBDlg->encrypted->Text;

   delete NymGenRBDlg;

   return mrResult==mrOk;
}
//---------------------------------------------------------------------------

__fastcall TNymGenRBDlg::TNymGenRBDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TNymGenRBDlg::TNymGenRBDlg(TComponent* Owner, TStrings *Blocks, TPGPsdk *Pgp,
   bool Preview) : TForm(Owner)
{
   assert(Owner != NULL && Blocks != NULL);

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, NULL, NULL);

   blocks = new TStringList();
   for (int i = 0; i < Blocks->Count; i++)
   {
      AnsiString line = Blocks->Strings[i].Trim();
      if (!line.IsEmpty())
         blocks->Add(line);
   }

   block = new TStringList();
   leg = new TStringList();
   names = new TStringList();
   remailers = new TStringList();
   encrypted = new TStringList();
   encblock = new TStringList();
   dead = new TStringList();
   message = new TStringList();
   dirs = new TStringList();
   hdrs = new TStringList();

   preview = Preview;
   PGP = Pgp;
}
//---------------------------------------------------------------------------

__fastcall TNymGenRBDlg::~TNymGenRBDlg()
{
   delete hdrs;
   delete dirs;
   delete message;
   delete dead;
   delete encblock;
   delete encrypted;
   delete remailers;
   delete names;
   delete leg;
   delete block;
   delete blocks;
}
//---------------------------------------------------------------------------

void __fastcall TNymGenRBDlg::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TNymGenRBDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;

   if (GenerateReplyBlocks())
   {
      ModalResult = mrOk;
      return;
   }

   ModalResult = mrCancel;
}
//---------------------------------------------------------------------------

void __fastcall TNymGenRBDlg::GetRListRemailers(TStrings *remailers, const AnsiString &rlisttxt)
{
   assert(remailers != NULL);

   remailers->Clear();

   if (FileExists(rlisttxt))
   {
      TStringList *rlist = new TStringList();
      TStringList *current = new TStringList();

      rlist->LoadFromFile(rlisttxt);
      for (int i = 0; i < rlist->Count; i++)
      {
         AnsiString line = rlist->Strings[i];
         if (line.Pos("%") > 0)
            current->Add(GetTok(0, line, ' '));
      }

      for (int i = 0; i < rlist->Count; i++)
      {
         AnsiString line = rlist->Strings[i];
         if (line.Pos("$remailer{") == 1)
         {
            remailers->Add(
               GetTok(1, line, '"') + "=" + GetTok(0, GetTok(1, line, '<'), '>')
            );
         }
      }

      delete current;
      delete rlist;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymGenRBDlg::GetRemailerNamesFromBlocks(TStrings *blocks, TStrings *names)
{
   assert(blocks != NULL && names != NULL);

   for (int i = 0; i < blocks->Count; i++)
   {
      AnsiString line = blocks->Strings[i].Trim();
      if (line.IsEmpty()) continue;

      if (!line.Pos(":") && !line.Pos("."))
      {
         names->Add(line);
         continue;
      }

      int p;
      if ((p = line.Pos("-To: ")) > 0)
      {
         AnsiString name, data = line.SubString(p + 5, 256);
         p = 0; while ((name = GetTok(p, data, ',')) != "")
         {
            if (name.Pos("@") == 0)
               names->Add(name);
            p++;
         }
         continue;
      }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TNymGenRBDlg::CheckNamesForDeadRemailers(TStrings *remailers, TStrings *names)
{
   assert(remailers != NULL && names != NULL);

   dead->Clear();
   for (int i = 0; i < names->Count; i++)
   {
      AnsiString name = names->Strings[i];
      if (remailers->Values[name].IsEmpty())
         dead->Add(name);
   }

   if (dead->Count)
   {
      AnsiString msg1 =
         "You have 1 dead remailer in the reply blocks for this nym.       \n"
         "That remailers is:\n\n";
      AnsiString msg2 =
         "You have " + IntToStr(dead->Count) + " dead remailers in the reply blocks for this nym.      \n"
         "Those remailers are:\n\n";
      MsgBox(this, (dead->Count == 1 ? msg1 : msg2) + ""
         "" + dead->Text + "\n"
         "I'll stop the encryption now and give you the opportunity\n"
         "to correct the problem.\n\n"
         "If you are looking for the easiest solution, just delete any\n"
         "reply blocks containing that remailer and use the Reply\n"
         "Block Wizard create new ones.",
          MT_INFO, MB_OK);
      return false;
   }

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TNymGenRBDlg::ChangeNamesToAddresses(TStrings *blocks, TStrings *addresses, TStrings *names)
{
   assert(blocks != NULL && addresses != NULL && names != NULL);

   AnsiString text = blocks->Text;
   for (int i = 0; i < names->Count; i++)
   {
      AnsiString name = names->Strings[i];
      int p, x = 1; while ((p = text.SubString(x, 102400).Pos(name)) > 0)
      {
         text.Delete(x + p - 1, name.Length());
         AnsiString address = addresses->Values[name];
         text.Insert(address, x + p - 1);
         x = x + p - 1 + address.Length();
      }
   }
   blocks->Text = text;
}
//---------------------------------------------------------------------------

bool __fastcall TNymGenRBDlg::GenerateReplyBlocks()
{
   encrypted->Clear(); encblock->Clear(); remailers->Clear(); names->Clear();

   GetRListRemailers(remailers, MixDir() + "rlist.txt");
   GetRemailerNamesFromBlocks(blocks, names);
   if (!CheckNamesForDeadRemailers(remailers, names))
   {
      return false;
   }

   ChangeNamesToAddresses(blocks, remailers, names);

   while (GetNextBlock(block, blocks))
   {

      while (GetPrevLeg(leg, block))
      {
         if (!EncryptLeg(leg, encblock))
         {
            return false;
         }
      }
      assert(block->Count > 0);

      encrypted->Add(block->Strings[0]);
      encrypted->AddStrings(encblock);
      encrypted->Add("");
      encblock->Clear();
   }

   return true;
}
//---------------------------------------------------------------------------

/*bool __fastcall TNymGenRBDlg::GenerateReplyBlocks()
{
   encrypted->Clear(); remailers->Clear(); names->Clear();

   GetRListRemailers(remailers, MixDir() + "rlist.txt");
   GetRemailerNamesFromBlocks(blocks, names);
   if (!CheckNamesForDeadRemailers(remailers, names))
   {
      return false;
   }
   ChangeNamesToAddresses(blocks, remailers, names);

   while (GetNextBlock(block, blocks))
   {

      while (GetPrevLeg(leg, block))
      {
         if (!EncryptLeg(leg, encrypted))
         {
            return false;
         }
      }

      assert(block->Count > 0);
      encrypted->Insert(0, block->Strings[0]);
   }


   return true;
}*/
//---------------------------------------------------------------------------

bool __fastcall TNymGenRBDlg::GetNextBlock(TStrings *block, TStrings *blocks)
{
   assert(block != NULL && blocks != NULL);

   block->Clear();
   while (blocks->Count > 0)
   {
      AnsiString line = blocks->Strings[0];
      if (block->Count && line.Pos("Reply-Block:") == 1)
         break;

      blocks->Delete(0);
      block->Add(line);
   }

   return block->Count > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TNymGenRBDlg::GetPrevLeg(TStrings *leg, TStrings *block)
{
   assert(leg != NULL && block != NULL);

   leg->Clear();

   while (block->Count > 2)
   {
      int idx = block->Count - 2;

      AnsiString line = block->Strings[idx];
      block->Delete(idx);
      leg->Insert(0, line);

      if (line.Pos(":") == 0)
         break;
   }

   return leg->Count > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymGenRBDlg::GetLegDirectives(TStrings *leg, TStrings *dirs)
{
   assert(leg != NULL && dirs != NULL);

   dirs->Clear();
   for (int i = 1; i < leg->Count; i++)
   {
      AnsiString line = leg->Strings[i];
      AnsiString name = line.SubString(1, line.Pos(":"));

      if (directives.Pos("," + name + ",") > 0)
      {
         dirs->Add(line);
      }
   }

   assert(dirs->Count > 0);
}
//---------------------------------------------------------------------------

void __fastcall TNymGenRBDlg::GetLegHeaders(TStrings *leg, TStrings *hdrs)
{
   assert(leg != NULL && hdrs != NULL);

   hdrs->Clear();
   for (int i = 1; i < leg->Count; i++)
   {
      AnsiString line = leg->Strings[i];
      AnsiString name = line.SubString(1, line.Pos(":"));

      if (headers.Pos("," + name + ",") > 0)
      {
         hdrs->Add(line);
      }
   }
}
//---------------------------------------------------------------------------

bool __fastcall TNymGenRBDlg::IsFirstLeg(TStrings *Leg)
{
   assert(Leg->Count > 0);
   return Leg->Strings[0].Pos(" {") > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TNymGenRBDlg::EncryptLeg(TStrings *leg, TStrings *encrypted)
{
   assert(leg != NULL && encrypted != NULL);

   hdrs->Clear();
   dirs->Clear();
   message->Clear();
   AnsiString userid = leg->Strings[0]; // remailer name

   GetLegDirectives(leg, dirs);
   GetLegHeaders(leg, hdrs);

   // add directives
   message->Add("::");
   for (int i = 0; i < dirs->Count; i++)
      message->Add(dirs->Strings[i]);
   message->Add("");

   // add headers--only for last leg (first compiled)
   if (encrypted->Count == 0 && hdrs->Count > 0)
   {
      message->Add("##");
      for (int i = 0; i < hdrs->Count; i++)
         message->Add(hdrs->Strings[i]);
      message->Add("");
   }

   // add earlier compiled layer(s)
   if (encrypted->Count > 0)
   {
      message->Add("::");
      message->Add("Encrypted: PGP");
      message->Add("");
      message->AddStrings(encrypted);
      encrypted->Clear();
      message->Add("");
   }

   // finito for message construction
   message->Add("**");

   if (preview)
      PreviewConfig(this, message);

   if (IsFirstLeg(leg) || // or compile leg...
       PGP->EncryptToUserID(PGPRemailerRing(), userid, message))
   {
      encrypted->AddStrings(message);
      return true;
   }

   return false;
}
//---------------------------------------------------------------------------
