//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "NymProfiles.h"

//---------------------------------------------------------------------------
// TNymProfile
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetNym()
{
   if (Count == 0)
      return "";

   //assert(Count > 0);

   AnsiString nym;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];
      if (line.Pos("Nym: ") == 1)
      {
         nym = line.SubString(6, 128);
         break;
      }
   }
   return nym;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetNymCommands()
{
   return "name=\"" + Name +"\" " + GetOptions();
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetNymIndexStr()
{
   if (Count == 0)
      return "";
      
   AnsiString item = GetNym();
   item += "\255" + GetName();
   item += "\255" + GetTok(0, GetPGPKey(), ' ');

   AnsiString opts = GetOptions();

   item += "\255" + opts.SubString(opts.Pos("acksend") - 1, 1);
   item += "\255" + opts.SubString(opts.Pos("signsend") - 1, 1);
   item += "\255" + opts.SubString(opts.Pos("cryptrecv") - 1, 1);
   item += "\255" + opts.SubString(opts.Pos("fixedsize") - 1, 1);
   item += "\255" + opts.SubString(opts.Pos("disable") - 1, 1);
   item += "\255" + opts.SubString(opts.Pos("nobcc") - 1, 1);
   item += "\255" + opts.SubString(opts.Pos("fingerkey") - 1, 1);

   item += GetRetired() ? "\255x" : "\255 ";
   return item;
}

//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetNymServer()
{
   AnsiString nym = GetNym();
   return nym.SubString(nym.Pos("@") + 1, 128);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetUserId()
{
   AnsiString nym = GetNym();
   return nym.SubString(1, nym.Pos("@") - 1);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetName()
{
   assert(Count > 0);

   AnsiString name;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];
      if (line.Pos("Name: ") == 1)
      {
         name = line.SubString(7, 128);
         break;
      }
   }
   return name;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetPGPKey()
{
   assert(Count > 0);

   AnsiString key;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];
      if (line.Pos("PgpKey: ") == 1)
      {
         key = line.SubString(9, 128);
         break;
      }
   }
   return key;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetOptions()
{
   assert(Count > 0);

   AnsiString opts;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];
      if (line.Pos("Commands: ") == 1)
      {
         opts += line.Pos("+acksend") > 0 ? "+acksend" : "-acksend";
         opts += line.Pos("+signsend") > 0 ? " +signsend" : " -signsend";
         opts += line.Pos("+cryptrecv") > 0 ? " +cryptrecv" : " -cryptrecv";
         opts += line.Pos("+fixedsize") > 0 ? " +fixedsize" : " -fixedsize";
         opts += line.Pos("+disable") > 0 ? " +disable" : " -disable";
         opts += line.Pos("+nobcc") > 0 ? " +nobcc" : " -nobcc";
         opts += line.Pos("+fingerkey") > 0 ? " +fingerkey" : " -fingerkey";
         opts = line.SubString(11, 128);
         break;
      }
   }
   return opts;
}
//---------------------------------------------------------------------------

/*void __fastcall TNymProfile::GetPublicKey(TStrings *Key)
{
   assert(Key != NULL && Count > 0);
   int blank = 0;

   Key->Clear();
   bool saving = false;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];
      if (line.Pos("-----BEGIN PGP PUBLIC KEY BLOCK-----") == 1)
         saving = true;

      if (saving && line.IsEmpty() && ++blank == 2)
         break;

      if (saving)
         Key->Add(line);

      if (line.Pos("-----END PGP PUBLIC KEY BLOCK-----") == 1)
         break;
   }
}*/
//---------------------------------------------------------------------------

void __fastcall TNymProfile::GetPublicKey(TStrings *Key)
{
   assert(Key != NULL && Count > 0);
   int blank = 0;

   Key->Clear();
   bool saving = false;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];
      if (line.Pos("Public Key:") == 1)
         saving = true;

      if (saving && line.IsEmpty() && ++blank == 2)
         break;

      if (saving)
         Key->Add(line);

      if (line.Pos("-----END PGP PUBLIC KEY BLOCK-----") == 1)
         break;
   }

   // fix cases where a bug has 2 "Public Key:" lines
   assert(Key->Count > 1);
   if (Key->Strings[1].Pos("Public Key:") == 1)
      Key->Delete(1);
}
//---------------------------------------------------------------------------

void __fastcall TNymProfile::GetPublicKeyNoKeyID(TStrings *Key)
{
   assert(Key != NULL && Count > 0);
   int blank = 0;

   Key->Clear();
   bool saving = false;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];
      if (line.Pos("-----BEGIN PGP PUBLIC KEY BLOCK-----") == 1)
         saving = true;

      if (saving && line.IsEmpty() && ++blank == 2)
         break;

      if (saving)
         Key->Add(line);

      if (line.Pos("-----END PGP PUBLIC KEY BLOCK-----") == 1)
         break;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymProfile::GetSecretKey(TStrings *Key)
{
   assert(Key != NULL && Count > 0);
   int blank = 0;

   Key->Clear();
   bool saving = false;

   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];

      if (line.Pos("Secret Key:") == 1)
         saving = true;

      if (saving && line.IsEmpty() && ++blank == 2)
         break;

      if (saving)
         Key->Add(line);

      if (line.Pos("-----END PGP PRIVATE KEY BLOCK-----") == 1)
         break;
   }

   // fix cases where a bug has 2 "Secret Key:" lines
   assert(Key->Count > 1);
   if (Key->Strings[1].Pos("Secret Key:") == 1)
      Key->Delete(1);
}
//---------------------------------------------------------------------------

void __fastcall TNymProfile::GetSecretKeyNoKeyID(TStrings *Key)
{
   assert(Key != NULL && Count > 0);
   int blank = 0;

   Key->Clear();
   bool saving = false;

   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];

      if (line.Pos("-----BEGIN PGP PRIVATE KEY BLOCK-----") == 1)
         saving = true;

      if (saving && line.IsEmpty() && ++blank == 2)
         break;

      if (saving)
         Key->Add(line);

      if (line.Pos("-----END PGP PRIVATE KEY BLOCK-----") == 1)
         break;
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetPublicKeyID()
{
   assert(Count > 0);

   AnsiString line, keyID;
   for (int i = 0; i < Count; i++)
   {
      line = Strings[i];
      if (line.Pos("Public Key: ") == 1)
      {
         keyID = line.SubString(13, 10);
         break;
      }
   }
   return keyID;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymProfile::GetSecretKeyID()
{
   assert(Count > 0);

   AnsiString line, keyID;
   for (int i = 0; i < Count; i++)
   {
      line = Strings[i];
      if (line.Pos("Secret Key: ") == 1)
      {
         keyID = line.SubString(13, 10);
         break;
      }
   }
   return keyID;
}
//---------------------------------------------------------------------------

int __fastcall TNymProfile::GetReplyBlocks(TStrings *Blocks)
{
   assert(Blocks != NULL && Count > 0);

   Blocks->Clear();
   bool saving = false;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i].TrimRight();

      if (line.Pos("Public Key: ") == 1)
         break;

      if (line.Pos("Reply-Block:") == 1)
         saving = true;

      if (saving)
         Blocks->Add(line);
   }

   TrimNullsAfter(Blocks);
   return Blocks->Count;
}
//---------------------------------------------------------------------------

bool __fastcall TNymProfile::GetRetired()
{
   assert(Count > 0);

   return Strings[0].Pos("Retired: ") == 1;
}
//---------------------------------------------------------------------------

void __fastcall TNymProfile::Retire()
{
   assert(Count > 0);

   Strings[0] = "Retired: " + FormatDateTime("yyyy-mm-dd hh:nn", Now());
   Log("Retired");
}
//---------------------------------------------------------------------------

void __fastcall TNymProfile::Log(const AnsiString &msg)
{
   assert(msg != "" && Count > 0);

   Add("");
   bool ok = false;
   AnsiString datetime = FormatDateTime("yyyy-mm-dd hh:nn", Now());

   int i = 0; while (i < Count)
   {
      if (Strings[i].Pos("Log: ") == 1)
      {
         while (i < Count)
         {
            if (Strings[i].Trim().IsEmpty())
            {
               Insert(i, datetime + "  " + msg);
               ok = true;
               break;
            }
            i++;
         }
      }

      if (ok)
         break;
      i++;
   }

   Delete(Count - 1);
}
//---------------------------------------------------------------------------

int __fastcall TNymProfile::GetHistory(TStrings *history)
{
   assert(history != NULL && Count > 0);

   int count = 0;
   history->Clear();

   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];

      if (line.Pos("Sent: ") == 1 || line.Pos("Log: ") == 1)
         break;

      if (line.Pos("Date: ") == 1 || line.Pos("Retired: ") == 1)
         count++;

      history->Add(line);
   }

   TrimNullsAfter(history);
   return count;
}
//---------------------------------------------------------------------------

int  __fastcall TNymProfile::GetHistory(int pos, TStrings *history)
{
   history->Clear();

   if (Count == 0)
      return 0;

   assert(history != NULL);// && Count > 0);

   int count = -1;
   bool saving = false;

   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];

      if (line.Pos("Sent:") == 1 || line.Pos("Log:") == 1)
         break;

      if (line.Pos("Date:") == 1 || line.Pos("Retired:") == 1)
         count++;

      if (count > pos)
         break;

      if (count == pos && !saving)
         saving = true;

      if (saving)
         history->Add(line);
   }

   TrimNullsAfter(history);
   return count;
}
//---------------------------------------------------------------------------

int  __fastcall TNymProfile::CountHistories()
{
   assert(Count > 0);

   int count = 0;
   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];

      if (line.Pos("Sent: ") == 1)
         break;

      if (line.Pos("Date: ") == 1 || line.Pos("Retired: ") == 1)
         count++;
   }

   return count;
}
//---------------------------------------------------------------------------

void __fastcall TNymProfile::GetReplay(TStrings *replay)
{
   assert(replay != NULL && Count > 0);

   replay->Clear();
   bool saving = false;

   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];

      if (line.Pos("Sent: ") == 1)
         saving = true;

      if (line.Pos("Log: ") == 1)
         break;

      if (saving)
         replay->Add(line);
   }

   TrimNullsAfter(replay);
}
//---------------------------------------------------------------------------

void __fastcall TNymProfile::GetLog(TStrings *log)
{
   assert(log != NULL && Count > 0);

   log->Clear();
   bool saving = false;

   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];

      if (line.Pos("Log: ") == 1)
         saving = true;

      if (line.Pos("Notes: ") == 1)
         break;

      if (saving)
         log->Add(line);
   }

   TrimNullsAfter(log);
}
//---------------------------------------------------------------------------

void __fastcall TNymProfile::GetNotes(TStrings *notes)
{
   assert(notes != NULL && Count > 0);

   notes->Clear();
   bool saving = false;

   for (int i = 0; i < Count; i++)
   {
      AnsiString line = Strings[i];

      if (line.Pos("Notes: ") == 1)
         saving = true;

      if (saving)
         notes->Add(line);
   }

   TrimNullsAfter(notes);
}

//---------------------------------------------------------------------------
// NYMReplyBlock
//---------------------------------------------------------------------------

/*__fastcall TNYMReplyBlock::TNYMReplyBlock()
   :  TStringList()
{
}
//---------------------------------------------------------------------------

void __fastcall TNYMReplyBlock::SetBlock(int BlockIdx, TStrings *Block)
{
   BlockIndex = BlockIdx;
   AddStrings(Block);
}
//---------------------------------------------------------------------------

bool __fastcall TNYMReplyBlock::GetLeg(int Index, TNYMReplyBlockLeg *Leg)
{
   int idx = 1;
   Leg->Clear();
   TrimNulls(this);

   int i = 0; while(++i < Count)
   {
      AnsiString line = Strings[i].Trim();
      if (idx == Index) Leg->Add(line);
      if (line.Pos(":") == 0) idx++;
      if (idx < Index) continue;
      if (idx > Index) break;
      //i++;
   }

   if (Leg->Count)
   {
      Leg->ReplyBlock = this;
      Leg->BlockIndex = BlockIndex;
      Leg->LegIndex = Index;
      //Leg->directives = directives;
      //Leg->headers = headers;
      //Leg->rlist = rlist;
   }

   return Leg->Count>0;
}
//---------------------------------------------------------------------------

int __fastcall TNYMReplyBlock::LegCount()
{
   int count=0;
   for (int i=0; i<Count; i++)
      if (Strings[i].Trim().Pos(":")==0) count++;
   return count;
}
//---------------------------------------------------------------------------

bool __fastcall TNYMReplyBlock::IsValid()
{
   bool ok=true;
   TNYMReplyBlockLeg *leg = new TNYMReplyBlockLeg();
   int i=0; while(GetLeg(++i,leg))
   {
       ok = leg->IsValid();
       if (!ok) break;
   }
   delete leg;
   return ok;
}
//---------------------------------------------------------------------------
// NYMReplyBlocks
//---------------------------------------------------------------------------
__fastcall TNYMReplyBlocks::TNYMReplyBlocks()
   :  TStringList()
{
   rlist = new TStringList();
   rlist->Sorted = true;
   //LoadType1Names(rlist);
   directives = new TStringList();
   directives->Sorted = true;
   //LoadCpkDirectives(directives);
   headers = new TStringList();
   headers->Sorted = true;
   //LoadCpkHeaders(headers);
}
//---------------------------------------------------------------------------

__fastcall TNYMReplyBlocks::TNYMReplyBlocks(TStrings *Profile)
   :  TStringList()
{
   assert(Profile != NULL);

   int i = 0;
   while (i < Profile->Count - 1)
   {
      if (i < Profile->Count - 1 && Profile->Strings[i+1].Pos("Reply-Block:") == 1)
         break;
      i++;
   }
   while (i < Profile->Count - 1)
   {
      AnsiString line = Profile->Strings[i+1];
      if (line.Pos("Public Key:") == 1 || line.Pos("-----BEGIN PGP") == 1)
         break;
      Add(line);
      i++;
   }
   TrimNulls(this);
   rlist = new TStringList();
   rlist->Sorted = true;
   //LoadType1Names(rlist);
   directives = new TStringList();
   directives->Sorted = true;
   //LoadCpkDirectives(directives);
   headers = new TStringList();
   headers->Sorted = true;
   //LoadCpkHeaders(headers);
   SetBlocks(Blocks);
}
//---------------------------------------------------------------------------

__fastcall TNYMReplyBlocks::~TNYMReplyBlocks()
{
   delete headers;
   delete directives;
   delete rlist;
}
//---------------------------------------------------------------------------

void __fastcall TNYMReplyBlocks::SetBlocks(TStrings *Blocks)
{
   int i=-1; while(++i<Blocks->Count)
      if (Blocks->Strings[i].Trim().IsEmpty()) Blocks->Delete(i);
      else i++;
   AddStrings(Blocks);
}
//---------------------------------------------------------------------------

bool __fastcall TNYMReplyBlocks::GetBlock(int Index, TNYMReplyBlock *Block)
{
   Block->Clear();
   AnsiString line;
   int i = -1, idx = 0;
   while(++i < Count)
   {
      line = Strings[i].Trim();
      if (line.Pos("Reply-Block:") == 1) idx++;
      if (idx == Index)
      {
         Block->Add(line);
         break;
      }
   }
   while (++i < Count)
   {
      line = Strings[i].Trim();
      if (line.IsEmpty()) continue;

      if (line.Pos("Reply-Block:") == 1) break;
      Block->Add(line);
   }
   if (Block->Count)
   {
      Block->BlockIndex = Index;
    //Block->directives = directives;
    //Block->headers = headers;
    //Block->rlist = rlist;
   }
   return Block->Count > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TNYMReplyBlocks::GetBlock(int Index, TNYMReplyBlock *Block)
{
   int idx=0;
   Block->Clear();
   int i = 0; while(i < Count)
   {
      AnsiString line = Strings[i].Trim();
      if (line.Pos("Reply-Block:") == 1) idx++;
      if (idx == Index) Block->Add(line);
      if (idx < Index) continue;
      if (idx > Index) break;
      i++;
   }
view(Block,Index);
   Block->BlockIndex = Index;
   //Block->directives = directives;
   //Block->headers = headers;
   //Block->rlist = rlist;
   return Block->Count > 0;
}
//---------------------------------------------------------------------------

int __fastcall TNYMReplyBlocks::BlockCount()
{
   int count=0;
   for (int i=0; i<Count; i++)
      if (Strings[i].Trim().Pos("Reply-Block:")==1 ) count++;
   return count;
}
//---------------------------------------------------------------------------

bool __fastcall TNYMReplyBlocks::AreValid()
{
   bool ok=true;
   TStringList *rlist = new TStringList();
   TNYMReplyBlock *block = new TNYMReplyBlock();
   int i=0; while(GetBlock(++i,block))
   {
      ok = block->IsValid();
      if (!ok) break;
   }
   delete block;
   delete rlist;
   return ok;
}
//---------------------------------------------------------------------------
// NYMReplyBlockLeg
//---------------------------------------------------------------------------
__fastcall TNYMReplyBlockLeg::TNYMReplyBlockLeg()
   : TStringList()
{
}
//---------------------------------------------------------------------------
bool __fastcall TNYMReplyBlockLeg::CheckBadRecipientMatch()
{
   bool ok=false;
   // compare the recipient (-To:) with value terminating the leg (last line)
   AnsiString last=Count?Strings[Count-1]:s("");
   for (int i=0; i<Count; i++)
      if (Strings[i].Pos("-To: "+last)>0) ok = true;
   if (!ok) ReplyBlockError(RBLOCK_BADRECIPIENTMATCH,BlockIndex,LegIndex,Count,this);
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TNYMReplyBlockLeg::CheckInvalidDirectiveOrHeader()
{
   AnsiString line,label;
   int pos; bool ok=true;
   for (int i=0; i<Count; i++)
   {
      line = Strings[i];
      if ((pos=line.Pos(":"))==0) continue;
      label = line.SubString(1,pos);
      if (!directives->Find(label,NULL) && !headers->Find(label,NULL))
      {
         ReplyBlockError(RBLOCK_BADDIRECTIVEORHEADER,BlockIndex,LegIndex,i+1,this,
            NULL,directives,headers);
         ok = false;
         break;
      }
   }
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TNYMReplyBlockLeg::CheckInvalidRemailerName()
{
   bool ok=true;
   AnsiString line,remname;
   // don't check for invalid remailer on last leg since it will be the final
   // destination--not a remailer
   if (LegIndex==ReplyBlock->LegCount()) return true;
   // check for invalid remailer name.
   for (int i=0; i<Count-1; i++)
   {
      line = Strings[i];
      if (line.Pos("Test-To:")==1 || line.Pos("Anon-Post-To:")==1) continue;
      if (line.Pos("-To: ")>0)
      {
         remname = line.SubString(line.Pos(" ")+1,128);
         if (!rlist->Find(remname,NULL) && !rlist->Find(remname+" *",NULL))
         {
            ReplyBlockError(RBLOCK_BADREMAILERNAME,BlockIndex,LegIndex,i+1,this,rlist);
            ok = false;
            break;
         }
      }
   }
   return ok;
}
//---------------------------------------------------------------------------
bool __fastcall TNYMReplyBlockLeg::CheckInvalidRecipient()
{
   if (LegIndex!=ReplyBlock->LegCount()) return true;
   int i=-1; while(++i<Count)
   {
      AnsiString line = Strings[i];
      if (line.Pos("Anon-Post-To: ")==1) return true;
      if (line.Pos("-To: ")>0 && (!line.Pos("@") || !line.Pos(".")))
      {
         ReplyBlockError(RBLOCK_BADRECIPIENTADDRESS,BlockIndex,LegIndex,i+1,this,rlist);
         return false;
      }
   }
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall TNYMReplyBlockLeg::IsValid()
{
   bool ok=CheckInvalidRemailerName();
   if (ok) ok=CheckInvalidDirectiveOrHeader();
   if (ok) ok=CheckBadRecipientMatch();
   if (ok) ok=CheckInvalidRecipient();
   return ok;
}*/
//---------------------------------------------------------------------------


