//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "QSnymWins.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

#include "Global2.h"
#include "NymFrames2.h"
#include "QSnymVer.h"
#include "Images256s.h"
#include "AboutDlgs2.h"
#include "NymOptionsDlgs.h"
#include "HistoryWins.h"

HWND HelpWin = 0;
const AnsiString AppHelpFile = ChangeFileExt(Application->ExeName, ".chm");
TQSnymWin *QSnymWin;
//---------------------------------------------------------------------------

__fastcall TQSnymWin::TQSnymWin(TComponent* Owner)
        : TForm(Owner)
{
   if (!FileExists(AppDat()))
   {
      if (!FileExists(AppPGP()))
      {
         ShowMessage(
            "Your QS database (QS.dbc) is missing! Without \n"
            "it I can't continue!");
      }
      else
      {
         ShowMessage(
            "I find your QS datafile is encrypted. You need  \n"
            "to start QS and decrypt the file.");
      }

      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   if (!FileExists(AppDir() + "qspgp.bpl"))
   {
      ShowMessage(
         "I see you do not have QSpgp.bpl installed.  \n"
         "Without it I can't continue!"
      );
      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   if (!InitGlobal(this))
   {
      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      return;
   }

   Global->WM_QSAPPMESSAGE = WM_QSAPPMESSAGE;

   TIniFile *ini = new TIniFile(AppIni());

   if (ParamCount() > 0 && ParamStr(1).LowerCase().Pos("version") > 0)
   {
      ini->WriteString(NymIni, "version", ReleaseVersion);
      Application->ShowMainForm = false;
      PostMessage(Handle, WM_QUIT, 0, 0);
      delete ini;
      return;
   }

   Caption = "QuickSilver Nym Manager " + ReleaseVersion;
   Bevel1->Align = alTop;
   NavBar->Align = alTop;
   Panel2->Align = alClient;
   AnsiString path = "/QuickSilver/Nyms";

   Images256 = new TImages256(this);

   Panel2->Color = (TColor)ini->ReadInteger(NymIni, QSWINDOWCOLOR, QSWINDOWCOLORDEF);
   StrToFont(ini->ReadString(NymIni, QSWINDOWFONT, QSWINDOWFONTDEF), Panel2->Font);
   // Set Panel2 font before init DetailFrame

   DetailFrame = (TNymsFrame2*)OpenNymsFrame2(this, Panel2, path, Global);

   DetailFrame->CoolBar->Parent = this;
   DetailFrame->ToolBar->Height = 30;
   DetailFrame->ToolBar->ButtonHeight = 28;
   DetailFrame->ToolButton14->Show();

   DetailFrame->ExitBtn->Hide();
   DetailFrame->ToolButton1->Hide();
   DetailFrame->ToolButton2->Hide();
   DetailFrame->ToolButton9->Hide();
   DetailFrame->ToolButton10->Hide();
   DetailFrame->ToolButton11->Hide();
   DetailFrame->ToolButton12->Hide();
   DetailFrame->ToolButton13->Hide();
   DetailFrame->ToolButton14->Hide();

   DetailFrame->GoMenu->Visible = false;
   DetailFrame->FileNewItem->Visible = false;
   DetailFrame->FileOpenItem->Visible = false;
   DetailFrame->ViewAddressBarItem->Visible = false;
   DetailFrame->ViewSideTreeItem->Visible = false;
   DetailFrame->ViewSideBarItem->Visible = false;
   DetailFrame->ToolsRemailersItem->Visible = false;
   DetailFrame->ToolsKeysMenu->Visible = false;

   Menu = DetailFrame->Menu;

   CurPath->Caption = path;

   TColor color;
   // let's get font for listview, while we have ini open
   LoadFonts(ini, NymIni, color, Font, DetailFrame->ListView->Font, NULL);
   DetailFrame->ListView->Color = color;

   DetailFrame->ViewToolBarItem->Checked = ini->ReadBool(NymIni, QSTOOLBAR, true);
   DetailFrame->ViewStatusBarItem->Checked = ini->ReadBool(NymIni, QSSTATUSBAR, true);
   DetailFrame->ViewRetiredItem->Checked = ini->ReadBool(NymIni, QSNYMRETIRED, true);
   NavBar->Visible = false; //ini->ReadBool(ProxyIni, QSNAVBAR, true);
   Bevel1->Visible = NavBar->Visible;

   DetailFrame->CoolBar->Visible = DetailFrame->ViewToolBarItem->Checked;
   StatusBar->Visible = DetailFrame->ViewStatusBarItem->Checked;

   LoadWindowPosition(this, NymIni, ini);

   delete ini;

   LoadHtmlHelp();
   Application->OnMessage = AppMessage;

   DetailFrame->LoadIndex(path);
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (Application->MainForm->Visible)// if not, global init failed
   {
      TIniFile *ini = new TIniFile(AppIni());
      ini->WriteBool(NymIni, QSTOOLBAR, DetailFrame->CoolBar->Visible);
    //ini->WriteBool(NymIni, QSNAVBAR, NavBar->Visible);
      ini->WriteBool(NymIni, QSSTATUSBAR, StatusBar->Visible);
      ini->WriteBool(NymIni, QSNYMRETIRED, DetailFrame->ViewRetiredItem->Checked);
      SaveWindowPosition(this, NymIni, ini);
      delete ini;

      DeleteNymsFrame2(DetailFrame);
      delete Images256;
   }

   ShutDownGlobal();

   UnloadHtmlHelp();
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::AppMessage(tagMSG &Msg, bool &Handled)
{
   if (Msg.message == WM_QSAPPMESSAGE)
   {
      switch (Msg.wParam)
      {
         case QSMSGID_SHUTDOWN:
            Close();
            Handled = true;
            return;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::WMOpenOptions(TMessage &)
{
   OpenNymOptionsDialog(this, DetailFrame, "QuickSilver Nym Manager Options");
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::WMOpenQuickStart(TMessage &)
{
   //
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::WMOpenHelpTopics(TMessage& Msg)
{
   AnsiString topic = AppHelpFile + ">qsnymhelp";
   HelpWin = HTMLHelp(Handle, topic.c_str(), HH_DISPLAY_TOPIC, NULL);
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::WMOpenHistory(TMessage& Msg)
{
   AnsiString history = "qsnym.history.txt";
   if (FileExists(AppDir() + history))
   {
      OpenApplicationHistory(this, DetailFrame->ListView->Font,
         DetailFrame->ListView->Color, AppDir() + history);
      return;
   }

   if (FileExists(AppDir() + "docs\\" + history))
   {
      OpenApplicationHistory(this, DetailFrame->ListView->Font,
         DetailFrame->ListView->Color, AppDir() + "docs\\" + history);
      return;
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::WMShowAboutBox(TMessage &)
{
   ShowAboutBox(this, AppName + "  v" + ReleaseVersion);
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::WMGetViewMenu(TMessage &Msg)
{
   int flags = 0;
   if (StatusBar->Visible) flags |= flagStatusBar;
   if (NavBar->Visible) flags |= flagAddressBar;
   Msg.Result = flags;
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::WMSetViewMenu(TMessage &Msg)
{
   DetailFrame->ViewStatusBarItem->Checked = !DetailFrame->ViewStatusBarItem->Checked;
   StatusBar->Visible = DetailFrame->ViewStatusBarItem->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TQSnymWin::WMCloseApp(TMessage &)
{
   Close();
}
//---------------------------------------------------------------------------

