//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "RBRemDlgs.h"
#include "RBEkxDlgs2.h"
#include "RBEsubDlgs.h"
#include "RBAdvDlgs.h"
#include "RBLtDlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TRBRemDlg *RBRemDlg;
//---------------------------------------------------------------------------

bool __fastcall OpenRBRemailerDialog(TForm *Owner, TStrings *Spec,
   const AnsiString &Caps, bool Last)
{
   assert(Spec != NULL && Spec->Count > 1);

   RBRemDlg = new TRBRemDlg(Owner);
   RBRemDlg->Spec = Spec;
   RBRemDlg->Caps = Caps;
   RBRemDlg->EsubBtn->Enabled = Last;
   RBRemDlg->ShowModal();
   bool ok = RBRemDlg->ModalResult == mrOk;
   delete RBRemDlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TRBRemDlg::TRBRemDlg(TComponent* Owner)
        : TForm(Owner)
{
   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, RemView->Font, NULL);
   RemView->Color = color;
}
//---------------------------------------------------------------------------

void __fastcall TRBRemDlg::FormCloseQuery(TObject *, bool &)
{
   Spec->Clear();
   Spec->Add(RemNode->Text);
   for (int i = 0; i < RemNode->Count; i++)
      Spec->Add(" " + RemNode->Item[i]->Text);
}
//---------------------------------------------------------------------------

void __fastcall TRBRemDlg::FormShow(TObject *)
{
   LateBtn->Enabled = HasCap("latent");
   EkxBtn->Enabled = HasCap("ek") || HasCap("ekx");

   RemView->Items->Clear();
   RemView->Items->BeginUpdate();

   RemNode = RemView->Items->AddChild(NULL, Spec->Strings[0]);
   for (int i = 1; i < Spec->Count; i++)
      RemView->Items->AddChild(RemNode, Spec->Strings[i].Trim());

   RemView->FullExpand();
   RemNode->MakeVisible();
   RemView->Items->EndUpdate();

   Snooze(1);

}
//---------------------------------------------------------------------------

bool __fastcall TRBRemDlg::HasCap(const AnsiString &Cap)
{
   return  (Caps.Pos(" " + Cap + " ") > 0);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TRBRemDlg::GetDirective(const AnsiString &Dir)
{
   AnsiString line, value;

   for (int i = 1; i < RemNode->Count; i++)
   {
      line = RemNode->Item[i]->Text;
      if (line.Pos(Dir) == 1)
      {
         value = line.SubString(Dir.Length() + 1, 64);
         break;
      }
   }

   return value;
}
//---------------------------------------------------------------------------

int __fastcall TRBRemDlg::DelDirective(const AnsiString &Dir)
{
   int idx = -1;
   for (int i = 1; i < RemNode->Count; i++)
   {
      TTreeNode *node = RemNode->Item[i];
      if (node->Text.Pos(Dir) == 1)
      {
         node->Delete();
         idx = i;
         break;
      }
   }

   return idx;
}
//---------------------------------------------------------------------------

int __fastcall TRBRemDlg::SetDirective(const AnsiString &Dir,
   const AnsiString &Value)
{
   int idx = RemNode->Count;

   for (int i = 0; i < RemNode->Count; i++)
   {
      if (RemNode->Item[i]->Text.Pos(Dir) == 1)
      {
         RemNode->Item[i]->Text = Dir.Trim() + " " + Value;
         idx = i;
         break;
      }
   }

   if (idx == RemNode->Count)
      RemNode->Owner->AddChild(RemNode, Dir.Trim() + " " + Value);

   return idx;
}
//---------------------------------------------------------------------------

void __fastcall TRBRemDlg::LateBtnClick(TObject *)
{
   AnsiString latenttime = GetDirective("Latent-Time: ");
   if (latenttime.IsEmpty())
      latenttime = "+1:00r";

   if (OpenLatentTimeDialog(this, latenttime))
   {
      SetDirective("Latent-Time: ", latenttime);
   }
}
//---------------------------------------------------------------------------

void __fastcall TRBRemDlg::EsubBtnClick(TObject *)
{
   AnsiString key = GetDirective("Encrypt-Subject: "),
      sub = GetDirective("Subject: ");

   if (OpenRBEsubDialog(this, key, sub))
   {
      SetDirective("Encrypt-Subject: ", key);
      SetDirective("Subject: ", sub);
   }
}
//---------------------------------------------------------------------------

void __fastcall TRBRemDlg::EkxBtnClick(TObject *)
{
   AnsiString KEYPasswd, DESPasswd, CASPasswd;

   KEYPasswd = GetDirective("Encrypt-Key: ");
   DESPasswd = GetDirective("Encrypt-3DES: ");
   CASPasswd = GetDirective("Encrypt-CAST: ");

   if (OpenEncryptKeyDialog(this, KEYPasswd, DESPasswd, CASPasswd, Caps))
   {
      if (!CASPasswd.IsEmpty())
      {
         DelDirective("Encrypt-CAST: ");
         SetDirective("Encrypt-CAST: ", CASPasswd);
      }
      if (!DESPasswd.IsEmpty())
      {
         DelDirective("Encrypt-3DES: ");
         SetDirective("Encrypt-3DES: ", DESPasswd);
      }
      if (!KEYPasswd.IsEmpty())
      {
         DelDirective("Encrypt-Key: ");
         SetDirective("Encrypt-Key: ", KEYPasswd);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TRBRemDlg::AdvancedBtnClick(TObject *)
{
   OpenRBAdvancedDialog(this);
}
//---------------------------------------------------------------------------

