//---------------------------------------------------------------------------
#pragma hdrstop

#include "Global2.h"
#include "ValidNymKeys.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

bool __fastcall ValidKeyForNymserver(TForm *Owner, const AnsiString &Key,
   const AnsiString &Nymserver)
{
   TPGPKeyDesc *key = new TPGPKeyDesc(Owner, Key);
   bool ok = key->ValidForNymserverSilent(Nymserver);
   delete key;

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall CheckValidKeyForProfile(TForm *Owner, TPGPsdk *PGP, TNymProfile *profile)
{
   AnsiString keydesc, keyid = GetTok(0, profile->PGPKey, ' ');

   PGP->GetKeyLV(PGPPublicRing(), PGPSecretRing(), keyid, keydesc);

   TPGPKeyDesc *key = new TPGPKeyDesc(Owner, keydesc);
   bool ok = key->ValidForNymserverErrorMsg(profile->NymServer);
   delete key;

   return ok;
}

//---------------------------------------------------------------------------
// TPGPKeyDesc
//---------------------------------------------------------------------------

__fastcall TPGPKeyDesc::TPGPKeyDesc(TForm *Owner, const AnsiString &KeyDesc)
   : TObject()
{

   owner = Owner;
   key = KeyDesc;
}
//---------------------------------------------------------------------------

bool __fastcall TPGPKeyDesc::ValidForNymserverSilent(const AnsiString &Nymserver)
{
   GetCaps(Nymserver);
   if (HasCap("newnym"))
   {
      AnsiString algo = GetTok(0, KeyAlgo(), ' ');
      int bits = KeyBits();

      if (!KeySecret())
      {
         return false;
      }

      if (algo != "RSA legacy key pair" || bits < 0 || bits > 2048)
      {
         return false;
      }
   }
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TPGPKeyDesc::ValidForNymserverErrorMsg(const AnsiString &Nymserver)
{
   GetCaps(Nymserver);
   if (HasCap("newnym"))
   {
      AnsiString algo = KeyAlgo();//GetTok(0, KeyAlgo(), ' ');
      int bits = KeyBits();

      if (!KeySecret())
      {
         MsgBox(owner,
            "There is a problem with your nym's PGP key. It's public key is present,    \n"
            "but the secret key is missing. You probably exported it from your main     \n"
            "PGP program without including the secret key. Try again. Tick the box     \n"
            "\"Include Private Keys\" in the PGP Export Key Dialog.",
             MT_ERROR, MB_OK);
         return false;
      }

      if (algo != "RSA legacy key pair" || bits < 0 || bits > 2048)
      {
         MsgBox(owner,
           "This key pair is not compatible with the nym server you wish to use.    \n"
           "" + Nymserver + " requires RSA legacy key pairs with no more than 2048    \n"
           "bits. Your's is " + algo + ", with " + IntToStr(bits) + " bits. You will have to    \n"
           "select a different key. Sorry",
            MT_ERROR, MB_OK);
         return false;
      }
   }
   return true;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPGPKeyDesc::KeyUserID()
{
   return GetTok(0, key, '\255');
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPGPKeyDesc::KeyID()
{
   return GetTok(1, key, '\255');
}
//---------------------------------------------------------------------------

int __fastcall TPGPKeyDesc::KeyBits()
{
   AnsiString bitstr = GetTok(3, key, '\255');
   int p = bitstr.Pos("/");

   if (p > 0) return bitstr.SubString(1, p - 1).ToIntDef(-1);
   return bitstr.ToIntDef(-1);
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPGPKeyDesc::KeyAlgo()
{
   return GetTok(4, key, '\255');
}
//---------------------------------------------------------------------------

bool __fastcall TPGPKeyDesc::KeySecret()
{
   return GetTok(6, key, '\255') == "1";
}
//---------------------------------------------------------------------------

int __fastcall TPGPKeyDesc::KeyVer()
{
   return GetTok(7, key, '\255').ToIntDef(-1);
}
//---------------------------------------------------------------------------

bool __fastcall TPGPKeyDesc::HasCap(const AnsiString &cap)
{
   return caps.Pos(" " + cap + " ") > 0;
}
//---------------------------------------------------------------------------

bool __fastcall TPGPKeyDesc::GetCaps(const AnsiString &nymserver)
{
   if (key.IsEmpty())
      return "";

   AnsiString nymserverfile = AppDir() + "nymserver.asc"; //NymserverFile;
   if (FileExists(nymserverfile))
   {
      TStringList *Caps = new TStringList();
      Caps->LoadFromFile(nymserverfile);

      for (int i = 0; i < Caps->Count; i++)
      {
         AnsiString line = Caps->Strings[i];
         //if (line.Pos("-----BEGIN PGP ") == 1)
         //   break;

         if (line.Pos("$nymserver{") == 1 && line.Pos(nymserver))
         {
            caps = GetTok(3, line, '\"') + " ";
            break;
         }
      }
      delete Caps;
   }

   return !caps.IsEmpty();
}
//---------------------------------------------------------------------------
